@@ -64,13 +64,39 @@ class Service(CoolNameable):
 
         self.backend = backend
         self.run_function = run_function
-        self._credentials = auth.default()[0]
-        self.publisher = pubsub_v1.PublisherClient(credentials=self._credentials, batch_settings=BATCH_SETTINGS)
+        self._publisher = None
+        self._credentials = None
         super().__init__(*args, **kwargs)
 
     def __repr__(self):
         return f"<{type(self).__name__}({self.name!r})>"
 
+    @property
+    def publisher(self):
+        """Get or instantiate the publisher client for the service. No publisher is instantiated until this property is
+        called for the first time. This allows checking for the `GOOGLE_APPLICATION_CREDENTIALS` environment variable to
+        be put off until it's needed.
+
+        :return google.cloud.pubsub_v1.PublisherClient:
+        """
+        if not self._publisher:
+            self._publisher = pubsub_v1.PublisherClient(credentials=self.credentials, batch_settings=BATCH_SETTINGS)
+
+        return self._publisher
+
+    @property
+    def credentials(self):
+        """Get or instantiate the Google Cloud credentials for the service. No credentials are instantiated until this
+        property is called for the first time. This allows checking for the `GOOGLE_APPLICATION_CREDENTIALS` environment
+        variable to be put off until it's needed.
+
+        :return google.auth.credentials.Credentials:
+        """
+        if not self._credentials:
+            self._credentials = auth.default()[0]
+
+        return self._credentials
+
     def serve(self, timeout=None, delete_topic_and_subscription_on_exit=False):
         """Start the service as a child, waiting to accept questions from any other Octue service using Google Pub/Sub
         on the same Google Cloud project. Questions are accepted, processed, and answered asynchronously.
@@ -82,7 +108,7 @@ class Service(CoolNameable):
         logger.info("Starting %r.", self)
 
         topic = Topic(name=self.id, namespace=OCTUE_NAMESPACE, service=self)
-        subscriber = pubsub_v1.SubscriberClient(credentials=self._credentials)
+        subscriber = pubsub_v1.SubscriberClient(credentials=self.credentials)
 
         subscription = Subscription(
             name=self.id,
@@ -131,7 +157,13 @@ class Service(CoolNameable):
         :raise Exception: if any exception arises during running analysis and sending its results
         :return None:
         """
-        data, question_uuid, forward_logs, parent_sdk_version = self._parse_question(question)
+        (
+            data,
+            question_uuid,
+            forward_logs,
+            parent_sdk_version,
+            allow_save_diagnostics_data_on_crash,
+        ) = self._parse_question(question)
 
         topic = answer_topic or self.instantiate_answer_topic(question_uuid)
         self._send_delivery_acknowledgment(topic)
@@ -173,6 +205,7 @@ class Service(CoolNameable):
                 input_manifest=data["input_manifest"],
                 analysis_log_handler=analysis_log_handler,
                 handle_monitor_message=functools.partial(self._send_monitor_message, topic=topic),
+                allow_save_diagnostics_data_on_crash=allow_save_diagnostics_data_on_crash,
             )
 
             if analysis.output_manifest is None:
@@ -207,6 +240,7 @@ class Service(CoolNameable):
         input_manifest=None,
         subscribe_to_logs=True,
         allow_local_files=False,
+        allow_save_diagnostics_data_on_crash=True,
         question_uuid=None,
         push_endpoint=None,
         timeout=86400,
@@ -220,6 +254,7 @@ class Service(CoolNameable):
         :param octue.resources.manifest.Manifest|None input_manifest: an input manifest of any datasets needed for the question
         :param bool subscribe_to_logs: if `True`, subscribe to the child's logs and handle them with the local log handlers
         :param bool allow_local_files: if `True`, allow the input manifest to contain references to local files - this should only be set to `True` if the child will be able to access these local files
+        :param bool allow_save_diagnostics_data_on_crash: if `True`, allow the input values and manifest (and its datasets) to be saved by the child if it fails while processing them
         :param str|None question_uuid: the UUID to use for the question if a specific one is needed; a UUID is generated if not
         :param str|None push_endpoint: if answers to the question should be pushed to an endpoint, provide its URL here; if they should be pulled, leave this as `None`
         :param float|None timeout: time in seconds to keep retrying sending the question
@@ -250,7 +285,7 @@ class Service(CoolNameable):
             topic=answer_topic,
             namespace=OCTUE_NAMESPACE,
             project_name=self.backend.project_name,
-            subscriber=pubsub_v1.SubscriberClient(credentials=self._credentials),
+            subscriber=pubsub_v1.SubscriberClient(credentials=self.credentials),
             push_endpoint=push_endpoint,
         )
         answer_subscription.create(allow_existing=True)
@@ -267,6 +302,7 @@ class Service(CoolNameable):
             question_uuid=question_uuid,
             forward_logs=str(int(subscribe_to_logs)),
             octue_sdk_version=pkg_resources.get_distribution("octue").version,
+            allow_save_diagnostics_data_on_crash=str(int(allow_save_diagnostics_data_on_crash)),
             retry=retry.Retry(deadline=timeout),
         )
 
@@ -298,7 +334,7 @@ class Service(CoolNameable):
                 f"Cannot pull from {subscription.path!r} subscription as it is a push subscription."
             )
 
-        subscriber = pubsub_v1.SubscriberClient(credentials=self._credentials)
+        subscriber = pubsub_v1.SubscriberClient(credentials=self.credentials)
 
         message_handler = OrderedMessageHandler(
             subscriber=subscriber,
@@ -414,7 +450,7 @@ class Service(CoolNameable):
         """Parse a question in the Google Cloud Pub/Sub or Google Cloud Run format.
 
         :param dict|Message question:
-        :return (dict, str, bool, packaging.version.Version|None):
+        :return (dict, str, bool, packaging.version.Version|None, bool):
         """
         try:
             # Parse question directly from Pub/Sub or Dataflow.
@@ -438,7 +474,15 @@ class Service(CoolNameable):
         except AttributeError:
             parent_sdk_version = None
 
+        try:
+            allow_save_diagnostics_data_on_crash = get_nested_attribute(
+                question,
+                "attributes.allow_save_diagnostics_data_on_crash",
+            )
+        except AttributeError:
+            allow_save_diagnostics_data_on_crash = False
+
         if parent_sdk_version:
             parent_sdk_version = packaging.version.parse(parent_sdk_version)
 
-        return data, question_uuid, forward_logs, parent_sdk_version
+        return data, question_uuid, forward_logs, parent_sdk_version, allow_save_diagnostics_data_on_crash
